/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 */
package org.enhydra.zeus.binder;

import java.io.IOException;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Vector;

// Zeus imports
import org.enhydra.zeus.Binder;
import org.enhydra.zeus.Binding;
import org.enhydra.zeus.binding.AtomicProperty;
import org.enhydra.zeus.binding.Container;
import org.enhydra.zeus.binding.ContainerProperty;
import org.enhydra.zeus.source.DTDSource;
import org.enhydra.zeus.util.CapitalizationUtils;

// DTDParser imports
import com.wutka.dtd.DTD;
import com.wutka.dtd.DTDAny;
import com.wutka.dtd.DTDAttribute;
import com.wutka.dtd.DTDCardinal;
import com.wutka.dtd.DTDContainer;
import com.wutka.dtd.DTDElement;
import com.wutka.dtd.DTDEmpty;
import com.wutka.dtd.DTDItem;
import com.wutka.dtd.DTDName;
import com.wutka.dtd.DTDPCData;

/**
 * <p>
 *  <code>DTDBinder</code> implements the <code>{@link Binder}</code>
 *    interface and allows generation of Zeus
 *    <code>{@link Binding}</code>s from a DTD.
 * </p>
 *
 * @author Brett McLaughlin
 * @version 1.0
 */
public class DTDBinder extends BaseBinder {

    /**
     * <p>
     *  This constructor takes in a <code>{@link DTDSource}</code>
     *    to read an XML Schema from and allow generation of the 
     *    <code>{@link Binding}</code>s from it.
     * </p>
     *
     * @param source <code>Source</code> to read input from.
     */
    public DTDBinder(DTDSource source) {
        super(source);
    }

    /**
     * <p>
     *  This is integral portion of the <code>Binder</code>. It
     *    is responsible for returning a Zeus representation of
     *    the set of constraints that this binding represents,
     *    resulting from the supplied DTD.
     * </p>
     *
     * @return <code>List</code> - the resultant
     *         <code>Binding</code>s from conversion of
     *         constraints.
     * @throws <code>IOException</code> when errors in reading
     *         input occur.     
     */
    public List getBindings() throws IOException {
        // Get the DTD
        DTD dtd = ((DTDSource)source).getDTD();
        
        // Create a list for the bindings
        List bindings = new LinkedList();
    
        Iterator i = dtd.getItemsByType(DTDElement.class).iterator();
        while (i.hasNext()) {
            DTDElement dtdElement = (DTDElement)i.next();
            bindings.add(convertToBinding(dtdElement));
        }
    
        return bindings;
    }
    
    /**
     * <p>
     *  This will take a single DTD element (in the form of a
     *    <code>DTDElement</code>) and convert it to a Zeus
     *    <code>Binding</code> object.
     * </p>
     *
     * @param dtdElement <code>DTDElement</code> to convert to
     *        Zeus <code>Binding</code>.
     * @return <code>Binding</code> - the converted Zeus <code>Binding</code>.
     */
    private Binding convertToBinding(DTDElement dtdElement) {
        ContainerProperty container = 
            new ContainerProperty(
                CapitalizationUtils.justInitialUpper(dtdElement.getName()));
            
        // Deal with the content of the DTDElement
        handleContent(container, dtdElement.getContent());
        
        // Deal with the attributes of the DTDElement
        handleAttributes(container, dtdElement);
    
        return container;
    }
    
    /**
     * <p>
     *  This will take an element from a DTD (in <code>DTDElement</code>
     *    form) and create Zeus <code>{@link AtomicProperty}</code> objects
     *    from its attributes, adding them to the supplied
     *    <code>{@link Container}</code>.
     * </p>
     *
     * @param container <code>Container</code> to add properties to.
     * @param dtdElement <code>DTDElement</code> to read attributes of.
     */
    private void handleAttributes(Container container, DTDElement dtdElement) {
        Hashtable elementAttributes = dtdElement.attributes;
        Enumeration e = elementAttributes.elements();
        while (e.hasMoreElements()) {
            DTDAttribute attribute = (DTDAttribute)e.nextElement();
            /**
             * Create the attribute; all DTD attributes are 
             *   simply Java <code>String</code>s, since
             *   there is no typing in DTDs.
             */
            AtomicProperty atomicProperty =
                new AtomicProperty(
                    CapitalizationUtils.allLower(attribute.getName()), "String");
            container.addProperty(atomicProperty);
        }        
    }

    /**
     * <p>
     *  This will take an element's content (in <code>DTDItem</code>
     *    form) and create Zeus <code>{@link AtomicProperty}</code> objects
     *    from its attributes, adding them to the supplied
     *    <code>{@link Container}</code>.
     * </p>
     *
     * @param conainer <code>Container</code> to add properties to.
     * @param content <code>DTDItem</code> with element's content.
     */
    private void handleContent(Container container, DTDItem content) {
        // Cycle through the various types of content
        if (content instanceof DTDEmpty) {       
            // We're done - this is an empty element
        } else if (content instanceof DTDPCData) {
            // Add a single String property
            String propertyName =
                new StringBuffer(CapitalizationUtils.allLower(container.getName()))
                    .append("Data")
                    .toString();
            AtomicProperty property = 
                new AtomicProperty(propertyName, "String");
            container.addProperty(property);
        } else if (content instanceof DTDName) {                                               
            // Create the correct type of property
            String propertyName = 
                CapitalizationUtils.allLower(((DTDName)content).getValue());
            DTDCardinal cardinality = ((DTDName)content).getCardinal();
            
            // Handle cardinality - either a list or a single property
            if ((cardinality.equals(DTDCardinal.NONE)) ||
                (cardinality.equals(DTDCardinal.OPTIONAL))) {
                
                // Get the type
                String propertyType =
                    CapitalizationUtils.justInitialUpper(((DTDName)content).getValue());
                
                // Generate a single property
                AtomicProperty property = 
                    new AtomicProperty(propertyName, propertyType);
                container.addProperty(property);
            } else {
            
                // Generate a list of properties
                AtomicProperty property =
                    new AtomicProperty(propertyName, "java.util.List");
                container.addProperty(property);
            }
        } else if (content instanceof DTDAny) {
            // XXX: Not sure WHAT to do here? Maybe a List?
            String propertyName =
                new StringBuffer(CapitalizationUtils.allLower(container.getName()))
                    .append("AnyData")
                    .toString();
            AtomicProperty property =
                new AtomicProperty(propertyName, "java.util.List");
        } else if (content instanceof DTDContainer) {       
            // Get each item and handle it
            Vector items = ((DTDContainer)content).getItemsVec();
            for (Iterator i = items.iterator(); i.hasNext(); ) {
                DTDItem subContent = (DTDItem)i.next();
                handleContent(container, subContent);
            }
        }    
    }

}
