/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 */
package org.enhydra.zeus.binding;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

/**
 * <p>
 *  <code>ContainerProperty</code> implements the 
 *    <code>{@link org.enhydra.zeus.Binding}</code>,
 *    <code>{@link Property}</code>, and
 *    <code>{@link Container}</code> interfaces, and provides
 *    for a complex object that is itself a property.
 * </p><p>
 *  A <code>ContainerProperty</code> will most commonly represent
 *    a Java <code>Object</code> that has properties of its own 
 *    (like any other <code>Container</code), but that also is a
 *    property itself, of some other Java <code>Object</code>.
 * </p>
 *
 * @author Brett McLaughlin
 * @version 1.0
 */
public class ContainerProperty extends BaseBinding implements Property, Container {

    /**
     * Note that since Java doesn't allow multiple inheritance, this class
     *   can't simple extend <code>{@link BaseProperty}</code> and
     *   <code>{@link BaseContainer}</code>, which is really what it should
     *   do. Instead, all those methods must be duplicated here, which is really
     *   sort of crummy.
     */

    /** The access level modifier for this <code>Property</code> */
    protected int accessModifier;
    
    /** The default value for this <code>Property</code> */
    protected Object defaultValue;    
    
    /** 
     * The <code>{@link Property}</code> objects this
     *  <code>Container</code> holds.
     */  
    protected List properties;    
    
    /**
     * <p>
     *  This will create a new <code>ContainerProperty</code> with
     *    the specified information.
     * </p>
     *
     * @param name the <code>String</code> name of this property.
     * @param accessModifier the <code>int<code> constant for the
     *        property's access modifier.
     * @see <code>{@link Property#ACCESS_PRIVATE}</code>
     * @see <code>{@link Property#ACCESS_PROTECTED}</code>
     * @see <code>{@link Property#ACCESS_PUBLIC}</code>          
     */    
    public ContainerProperty(String name, int accessModifier) {
        this.name = name;
        this.accessModifier = accessModifier;
    }
    
    /**
     * <p>
     *  This will create a new <code>ContainerProperty</code> with
     *    the specified information. The property will be, by default,
     *    <code>{@link Property#ACCESS_PRIVATE}</code>.
     * </p>
     *
     * @param name the <code>String</code> name of this property.        
     */    
    public ContainerProperty(String name) {
        this(name, Property.ACCESS_PRIVATE);
    }

    /**
     * <p>
     *  This will set the access level modifier for a property. The
     *    value submitted must be in the form of an <code>int</code>,
     *    which should correspond to one of the constants defined
     *    (<code>{@link #ACCESS_PRIVATE}</code>,
     *     <code>{@link #ACCESS_PROTECTED}</code>,
     *     or <code>{@link #ACCESS_PUBLIC}</code>. By default,
     *    all properties will be <code>private</code>
     *    (<code>ACCESS_PRIVATE</code>).
     * </p>
     *
     * @param modifier <code>int</code> constant for access level.
     * @see #ACCESS_PRIVATE
     * @see #ACCESS_PROTECTED
     * @see #ACCESS_PUBLIC          
     */
    public void setAccessModifier(int accessModifier) {
        this.accessModifier = accessModifier;
    }
    
    /**
     * <p>
     *  This will return the access level modifier for a property. The
     *    value returned will be in the form of an <code>int</code>,
     *    which will correspond to one of the constants defined
     *    (<code>{@link #ACCESS_PRIVATE}</code>,
     *     <code>{@link #ACCESS_PROTECTED}</code>,
     *     or <code>{@link #ACCESS_PUBLIC}</code>.
     * </p>
     *
     * @return <code>int</code> - constant for access level modifier.
     * @see #ACCESS_PRIVATE
     * @see #ACCESS_PROTECTED
     * @see #ACCESS_PUBLIC          
     */    
    public int getAccessModifier() {
        return accessModifier;    
    }
    
    /**
     * <p>
     *  This will return the Java <code>String</code> representation
     *    of this <code>Property</code>'s access modifier. For
     *    example, <code>{@link #ACCESS_PRIVATE}</code> would be
     *    converted to "private".
     * </p>
     *
     * @return <code>String</code> - Java representation
     * @see #ACCESS_PRIVATE
     * @see #ACCESS_PROTECTED
     * @see #ACCESS_PUBLIC               
     */
    public String getAccessModifierString() {
        if (accessModifier == ACCESS_PRIVATE) {
            return "private";
        } else if (accessModifier == ACCESS_PROTECTED) {
            return "protected";
        } else if (accessModifier == ACCESS_PUBLIC) {
            return "public";
        } else {
            throw new UnsupportedOperationException("Unknown access modifier");
        }
    }    
    
    /**
     * <p>
     *  This will set the default value of the property. Since no typing
     *    is available at this point, a simple Java <code>Object</code>
     *    is allowed as the type supplied. As a result, any errors in 
     *    mismatches between object type and allowed paramater type will
     *    occur at runtime, when class generation takes place. Supplying
     *    a value here essentially results in:
     * <code><pre>
     *    public class Foo {
     *
     *        private String myString = "some default value";
     *
     *        public String getMyString() {
     *            return myString;
     *        }
     *
     *        public void setMyString(String myString) {
     *            this.myString = myString;
     *        }
     * 
     *        // Other methods and properties
     *    }
     * </pre></code>
     * </p><p>
     *  Also, note that data binding users who supply their own class
     *    implementations will <b>LOSE THIS DEFAULT VALUE</b>, as the interface
     *    alone cannot specify a default value. So use this carefully!
     * </p>
     *
     * @param defaultValue <code>Object</code> to be used as default value.
     */
    public void setDefaultValue(Object defaultValue) {
        this.defaultValue = defaultValue;    
    }
    
    /**
     * <p>
     *  This will retrieve the default value associated with this property,
     *    or <code>null</code> if there is not one. For more information on
     *    default property values, see <code>{@link #setDefaultValue(Object)}</code>.
     * </p>
     *
     * @return <code>Object</code> - default value of the property.
     */
    public Object getDefaultValue() {
        return defaultValue;
    }
    
    /**
     * <p>
     *  This will add a <code>{@link Property}</code> to
     *    the member variables of this <code>Container</code>.
     * </p><p>
     *  It is important to note that this is simply an instance
     *    variable being added to (in most cases) a custom
     *    Java class definition. That variable/property
     *    doesn't have a value (although it might have a
     *    default value) until marshalling and unmarshalling
     *    occurs.
     * </p>
     *
     * @param property <code>Property</code> to add.
     */
    public void addProperty(Property property) {
        if (properties == null) {
            properties = new LinkedList();
        }
        
        properties.add(property);
    }
    
    /**
     * <p>
     *  This will remove a <code>{@link Property}</code> 
     *    from this <code>Container</code>, given the
     *    property's name. If a successful removal occurs,
     *    the boolean value <code>true</code> is retruned.
     *    If no <code>Property<code> is found with the supplied
     *    name, the boolean value <code>false</code> is returned.
     * </p>
     *
     * @param propertyName <code>String</code> name of 
     *        <code>Property</code> to remove.
     * @return <code>boolean</code> - indicates whether
     *         the specified named <code>Property</code>
     *         was found and removed.
     */
    public boolean removeProperty(String propertyName) {
        if (properties == null) {
            return false;
        }
        
        for (Iterator i = properties.iterator(); i.hasNext(); ) {
            Property property = (Property)i.next();
            if (property.getName().equals(propertyName)) {
                i.remove();
                return true;
            }
        }
        
        return false;
    }    
    
    /**
     * <p>
     *  This will return a list of all the 
     *    <code>{@link Property}</code> objects that
     *    this <code>Container</code> has. If there are
     *    none, this will return an empty <code>List</code>.
     * </p>
     *
     * @return <code>List</code> - properties for this
     *         <code>Container</code>.
     */
    public List getProperties() {
        if (properties == null) {
            return new LinkedList();
        }
        
        return properties;
    }    
    
    /**
     * <p>
     *  This will clear all the properties for this <code>Container</code>.
     * </p>
     */
    public void clearProperties() {
        if (properties != null) {
            properties.clear();
        }
    }    

}
